@@ -23,7 +23,7 @@ import univention.debug as ud
 from univention.management.console.config import ucr
 from univention.management.console.error import BadRequest, Forbidden, UMC_Error, Unauthorized
 from univention.management.console.ldap import get_machine_connection
-from univention.management.console.log import CORE
+from univention.management.console.log import CORE, RequestFilter
 from univention.management.console.session import Session
 
 
@@ -55,6 +55,7 @@ class Resource(RequestHandler):
         self.request.content_negotiation_lang = 'json'
         self.decode_request_arguments()
         await self.parse_authorization()
+        self._set_request_context()
         if not self.ignore_session_timeout_reset:
             self.current_user.reset_timeout()  # FIXME: order correct?
         await self.refresh_oidc_session()
@@ -63,6 +64,18 @@ class Resource(RequestHandler):
         if self.requires_authentication and not self.current_user.user.authenticated:
             raise Forbidden(self._("For using this request a login is required."))
 
+    def _set_request_context(self):
+        RequestFilter.request_context.set(self._request_context())
+
+    def _request_context(self):
+        session = self.current_user
+        return {
+            'request_id': '',
+            'requester_dn': session.user.user_dn,
+            'requester_ip': session.user.ip,
+            # 'requester_hostname': '',
+        }
+
     def check_session_validity(self):
         if not self.requires_authentication:  # TODO: or do we just want to disable this during auth?
             return
@@ -165,12 +178,12 @@ class Resource(RequestHandler):
         current_ip = self.current_user.user.ip
         # make sure a lost connection to the UMC-Server does not bind the session to ::1
         if ip != current_ip and any(ipaddress.ip_address(current_ip) in network for network in allowed_networks):
-            CORE.warn('Switching session IP from=%r to=%r' % (current_ip, ip))
+            CORE.warning('Switching session IP from=%r to=%r', current_ip, ip)
             self.current_user.user.ip = current_ip = ip
 
         # bind session to IP (allow requests from localhost)
         if ip != current_ip and not any(ipaddress.ip_address(ip) in network for network in allowed_networks):
-            CORE.warn('The sessionid (ip=%s) is not valid for this IP address (%s)' % (ip, current_ip))
+            CORE.warning('The sessionid (ip=%s) is not valid for this IP address (%s)', ip, current_ip)
             # very important! We must expire the session cookie, with the same path, otherwise one ends up in a infinite redirection loop after changing the IP address (e.g. because switching from VPN to regular network)
             for name in self.request.cookies:
                 if name.startswith('UMCSessionId'):
@@ -348,7 +361,7 @@ class Resource(RequestHandler):
             index = message.find('Traceback') if 'Traceback' in message else message.find('File')
             message, stacktrace = message[:index].strip(), message[index:].strip()
         if stacktrace:
-            CORE.error('%s', stacktrace)
+            CORE.error('Unexpected exception.', traceback=stacktrace)
         if ucr.is_false('umc/http/show_tracebacks', False):
             stacktrace = None
 
